@@ -28,7 +28,7 @@
  *
  */
 
-#include "cogl-config.h"
+#include "config.h"
 
 #include <string.h>
 #include <math.h>
@@ -92,15 +92,15 @@ static const CoglPixelFormatInfo format_info_table[] = {
     .bpp = { 0 },
   },
   {
-    .cogl_format = COGL_PIXEL_FORMAT_G_8,
-    .format_str = "G_8",
+    .cogl_format = COGL_PIXEL_FORMAT_R_8,
+    .format_str = "R_8",
     .n_planes = 1,
     .aligned = 1,
     .bpp = { 1 },
   },
   {
-    .cogl_format = COGL_PIXEL_FORMAT_G_16,
-    .format_str = "G_16",
+    .cogl_format = COGL_PIXEL_FORMAT_R_16,
+    .format_str = "R_16",
     .n_planes = 1,
     .aligned = 1,
     .bpp = { 2 },
@@ -119,6 +119,20 @@ static const CoglPixelFormatInfo format_info_table[] = {
     .aligned = 1,
     .bpp = { 4 },
   },
+  {
+    .cogl_format = COGL_PIXEL_FORMAT_RGBA_16161616,
+    .format_str = "RGBA_16161616",
+    .n_planes = 1,
+    .aligned = 1,
+    .bpp = { 8 },
+  },
+  {
+    .cogl_format = COGL_PIXEL_FORMAT_RGBA_16161616_PRE,
+    .format_str = "RGBA_16161616_PRE",
+    .n_planes = 1,
+    .aligned = 1,
+    .bpp = { 8 },
+  },
   {
     .cogl_format = COGL_PIXEL_FORMAT_RGB_888,
     .format_str = "RGB_888",
@@ -133,7 +147,7 @@ static const CoglPixelFormatInfo format_info_table[] = {
     .aligned = 1,
     .bpp = { 3 },
   },
-    {
+  {
     .cogl_format = COGL_PIXEL_FORMAT_RGBX_8888,
     .format_str = "RGBX_8888",
     .n_planes = 1,
@@ -385,6 +399,20 @@ static const CoglPixelFormatInfo format_info_table[] = {
     .bpp = { 8 },
     .aligned = 1
   },
+  {
+    .cogl_format = COGL_PIXEL_FORMAT_RGBA_FP_32323232,
+    .format_str = "RGBA_FP_32323232",
+    .n_planes = 1,
+    .bpp = { 16 },
+    .aligned = 1
+  },
+  {
+    .cogl_format = COGL_PIXEL_FORMAT_RGBA_FP_32323232_PRE,
+    .format_str = "RGBA_FP_32323232_PRE",
+    .n_planes = 1,
+    .bpp = { 16 },
+    .aligned = 1
+  },
   {
     .cogl_format = COGL_PIXEL_FORMAT_DEPTH_16,
     .format_str = "DEPTH_16",
@@ -392,13 +420,6 @@ static const CoglPixelFormatInfo format_info_table[] = {
     .aligned = 1,
     .bpp = { 2 },
   },
-  {
-    .cogl_format = COGL_PIXEL_FORMAT_DEPTH_32,
-    .format_str = "DEPTH_32",
-    .n_planes = 1,
-    .aligned = 1,
-    .bpp = { 4 },
-  },
   {
     .cogl_format = COGL_PIXEL_FORMAT_DEPTH_24_STENCIL_8,
     .format_str = "DEPTH_24_STENCIL_8",
