@@ -136,13 +136,19 @@ proxy_bell (MetaSeatImpl   *seat_impl,
 }
 
 static void
-proxy_mods_state_changed (MetaSeatImpl   *seat_impl,
-                          ClutterSeat    *seat)
+keymap_state_changed_cb (MetaSeatNative *seat_native,
+                         ClutterKeymap  *keymap)
 {
-  ClutterKeymap *keymap;
+  xkb_layout_index_t idx;
 
-  keymap = clutter_seat_get_keymap (seat);
-  g_signal_emit_by_name (keymap, "state-changed");
+  idx = clutter_keymap_get_layout_index (keymap);
+
+  if (idx != seat_native->xkb_layout_index)
+    {
+      seat_native->xkb_layout_index = idx;
+      meta_backend_notify_keymap_layout_group_changed (seat_native->backend,
+                                                       idx);
+    }
 }
 
 static void
@@ -161,8 +167,6 @@ meta_seat_native_constructed (GObject *object)
                     G_CALLBACK (proxy_touch_mode_changed), seat);
   g_signal_connect (seat->impl, "bell",
                     G_CALLBACK (proxy_bell), seat);
-  g_signal_connect (seat->impl, "mods-state-changed",
-                    G_CALLBACK (proxy_mods_state_changed), seat);
 
   seat->core_pointer = meta_seat_impl_get_pointer (seat->impl);
   seat->core_keyboard = meta_seat_impl_get_keyboard (seat->impl);
@@ -287,7 +291,14 @@ meta_seat_native_get_keymap (ClutterSeat *seat)
   MetaSeatNative *seat_native = META_SEAT_NATIVE (seat);
 
   if (!seat_native->keymap)
-    seat_native->keymap = meta_seat_impl_get_keymap (seat_native->impl);
+    {
+      seat_native->keymap = meta_seat_impl_get_keymap (seat_native->impl);
+      g_signal_connect_object (seat_native->keymap,
+                               "state-changed",
+                               G_CALLBACK (keymap_state_changed_cb),
+                               seat,
+                               G_CONNECT_SWAPPED);
+    }
 
   return CLUTTER_KEYMAP (seat_native->keymap);
 }
@@ -548,7 +559,7 @@ set_impl_keyboard_map_cb (GObject      *source_object,
 
   if (!meta_seat_impl_set_keyboard_map_finish (seat_impl, result, &error))
     {
-      g_task_return_error (task, error);
+      g_task_return_error (task, g_steal_pointer (&error));
       return;
     }
 
