@@ -12,17 +12,11 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
-#ifndef META_BACKEND_X11_TYPES_H
-#define META_BACKEND_X11_TYPES_H
-
+#pragma once
 typedef struct _MetaBackendX11 MetaBackendX11;
 
 typedef struct _MetaX11Barriers MetaX11Barriers;
-
-#endif /* META_BACKEND_X11_TYPES_H */
