@@ -29,8 +29,6 @@
 #include "core/display-private.h"
 #include "core/startup-notification-private.h"
 #include "core/util-private.h"
-#include "meta/meta-x11-errors.h"
-#include "x11/meta-x11-display-private.h"
 
 /* This should be fairly long, as it should never be required unless
  * apps or .desktop files are buggy, and it's confusing if
@@ -145,7 +143,7 @@ meta_startup_notification_update_cursor (MetaStartupNotification *sn)
     {
       meta_topic (META_DEBUG_STARTUP,
                   "Setting busy cursor");
-      cursor = META_CURSOR_BUSY;
+      cursor = META_CURSOR_WAIT;
     }
   else
     {
@@ -161,15 +159,13 @@ meta_startup_notification_update_cursor (MetaStartupNotification *sn)
     }
 }
 
-static gboolean
+static void
 meta_startup_notification_cursor_timeout (gpointer user_data)
 {
   MetaStartupNotification *sn = user_data;
 
   meta_startup_notification_update_cursor (sn);
   sn->update_cursor_timeout_id = 0;
-
-  return G_SOURCE_REMOVE;
 }
 
 static void
@@ -180,9 +176,9 @@ meta_startup_notification_update_feedback (MetaStartupNotification *sn)
 
   meta_startup_notification_update_cursor (sn);
   sn->update_cursor_timeout_id =
-    g_timeout_add (UPDATE_CURSOR_TIMEOUT_MS,
-                   meta_startup_notification_cursor_timeout,
-                   sn);
+    g_timeout_add_once (UPDATE_CURSOR_TIMEOUT_MS,
+                        meta_startup_notification_cursor_timeout,
+                        sn);
 }
 
 static void
@@ -320,58 +316,42 @@ meta_startup_sequence_class_init (MetaStartupSequenceClass *klass)
                   G_TYPE_NONE, 0);
 
   seq_props[PROP_SEQ_DISPLAY] =
-    g_param_spec_object ("display",
-                         "Display",
-                         "Display",
+    g_param_spec_object ("display", NULL, NULL,
                          META_TYPE_DISPLAY,
                          G_PARAM_READWRITE |
                          G_PARAM_CONSTRUCT_ONLY);
   seq_props[PROP_SEQ_ID] =
-    g_param_spec_string ("id",
-                         "ID",
-                         "ID",
+    g_param_spec_string ("id", NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE |
                          G_PARAM_CONSTRUCT_ONLY);
   seq_props[PROP_SEQ_TIMESTAMP] =
-    g_param_spec_uint64 ("timestamp",
-                         "Timestamp",
-                         "Timestamp",
+    g_param_spec_uint64 ("timestamp", NULL, NULL,
                          0, G_MAXUINT64, 0,
                          G_PARAM_READWRITE |
                          G_PARAM_CONSTRUCT_ONLY);
   seq_props[PROP_SEQ_ICON_NAME] =
-    g_param_spec_string ("icon-name",
-                         "Icon name",
-                         "Icon name",
+    g_param_spec_string ("icon-name", NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE |
                          G_PARAM_CONSTRUCT_ONLY);
   seq_props[PROP_SEQ_APPLICATION_ID] =
-    g_param_spec_string ("application-id",
-                         "Application ID",
-                         "Application ID",
+    g_param_spec_string ("application-id", NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE |
                          G_PARAM_CONSTRUCT_ONLY);
   seq_props[PROP_SEQ_WMCLASS] =
-    g_param_spec_string ("wmclass",
-                         "WM class",
-                         "WM class",
+    g_param_spec_string ("wmclass", NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE |
                          G_PARAM_CONSTRUCT_ONLY);
   seq_props[PROP_SEQ_WORKSPACE] =
-    g_param_spec_int ("workspace",
-                      "Workspace",
-                      "Workspace",
+    g_param_spec_int ("workspace", NULL, NULL,
                       G_MININT, G_MAXINT, -1,
                       G_PARAM_READWRITE |
                       G_PARAM_CONSTRUCT_ONLY);
   seq_props[PROP_SEQ_NAME] =
-    g_param_spec_string ("name",
-                         "Name",
-                         "Name",
+    g_param_spec_string ("name", NULL, NULL,
                          NULL,
                          G_PARAM_READWRITE |
                          G_PARAM_CONSTRUCT_ONLY);
@@ -657,6 +637,8 @@ meta_startup_notification_lookup_sequence (MetaStartupNotification *sn,
 static void
 meta_startup_notification_init (MetaStartupNotification *sn)
 {
+  sn->startup_sequences = NULL;
+  sn->startup_sequence_timeout_id = 0;
 }
 
 static void
@@ -711,31 +693,17 @@ meta_startup_notification_get_property (GObject    *object,
     }
 }
 
-static void
-meta_startup_notification_constructed (GObject *object)
-{
-  MetaStartupNotification *sn = META_STARTUP_NOTIFICATION (object);
-
-  g_assert (sn->display != NULL);
-
-  sn->startup_sequences = NULL;
-  sn->startup_sequence_timeout_id = 0;
-}
-
 static void
 meta_startup_notification_class_init (MetaStartupNotificationClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  object_class->constructed = meta_startup_notification_constructed;
   object_class->finalize = meta_startup_notification_finalize;
   object_class->set_property = meta_startup_notification_set_property;
   object_class->get_property = meta_startup_notification_get_property;
 
   sn_props[PROP_DISPLAY] =
-    g_param_spec_object ("display",
-                         "Display",
-                         "Display",
+    g_param_spec_object ("display", NULL, NULL,
                          META_TYPE_DISPLAY,
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
                          G_PARAM_STATIC_STRINGS);
