@@ -17,14 +17,16 @@
 
 #pragma once
 
+#include "backends/meta-backlight-private.h"
 #include "backends/native/meta-output-native.h"
 
 struct _MetaOutputTest
 {
-  MetaOutput parent;
+  MetaOutputNative parent;
 
   gboolean override_scale;
   float scale;
+  MetaBacklight *backlight;
 };
 
 #define META_TYPE_OUTPUT_TEST (meta_output_test_get_type ())
@@ -33,5 +35,16 @@ G_DECLARE_FINAL_TYPE (MetaOutputTest, meta_output_test,
                       META, OUTPUT_TEST,
                       MetaOutputNative)
 
+struct _MetaBacklightTest
+{
+  MetaBacklight parent;
+};
+
+#define META_TYPE_BACKLIGHT_TEST (meta_backlight_test_get_type ())
+META_EXPORT
+G_DECLARE_FINAL_TYPE (MetaBacklightTest, meta_backlight_test,
+                      META, BACKLIGHT_TEST,
+                      MetaBacklight)
+
 void meta_output_test_override_scale (MetaOutputTest *output_test,
                                       float           scale);
