@@ -244,6 +244,12 @@ meta_test_client_get_id (MetaTestClient *client)
   return client->id;
 }
 
+MetaWindowClientType
+meta_test_client_get_client_type (MetaTestClient *client)
+{
+  return client->type;
+}
+
 static void
 test_client_line_read (GObject      *source,
                        GAsyncResult *result,
@@ -310,46 +316,51 @@ meta_test_client_do_line (MetaTestClient  *client,
 }
 
 gboolean
-meta_test_client_dov (MetaTestClient  *client,
-                      GError         **error,
-                      va_list          vap)
+meta_test_client_do_strv (MetaTestClient  *client,
+                          const char     **args,
+                          GError         **error)
 {
-  GString *command = g_string_new (NULL);
-  GError *local_error = NULL;
+  g_autoptr (GString) command = NULL;
+  const char **arg;
 
-  while (TRUE)
+  command = g_string_new (NULL);
+  for (arg = &args[0]; *arg; arg++)
     {
-      char *word = va_arg (vap, char *);
-      char *quoted;
-
-      if (word == NULL)
-        break;
+      g_autofree char *quoted = NULL;
 
       if (command->len > 0)
         g_string_append_c (command, ' ');
 
-      quoted = g_shell_quote (word);
+      quoted = g_shell_quote (*arg);
       g_string_append (command, quoted);
-      g_free (quoted);
     }
 
   g_string_append_c (command, '\n');
 
-  if (!meta_test_client_do_line (client, command->str, &local_error))
-    goto out;
+  return meta_test_client_do_line (client, command->str, error);
+}
 
- out:
-  g_string_free (command, TRUE);
+gboolean
+meta_test_client_dov (MetaTestClient  *client,
+                      GError         **error,
+                      va_list          vap)
+{
+  g_autoptr (GStrvBuilder) args_builder = NULL;
+  g_auto (GStrv) args = NULL;
 
-  if (local_error)
+  args_builder = g_strv_builder_new ();
+  while (TRUE)
     {
-      g_propagate_error (error, local_error);
-      return FALSE;
-    }
-  else
-    {
-      return TRUE;
+      char *word = va_arg (vap, char *);
+
+      if (!word)
+        break;
+
+      g_strv_builder_add (args_builder, word);
     }
+
+  args = g_strv_builder_end (args_builder);
+  return meta_test_client_do_strv (client, (const char **) args, error);
 }
 
 gboolean
@@ -469,10 +480,20 @@ typedef struct _WaitForShownData
   gulong shown_handler_id;
 } WaitForShownData;
 
+static void
+window_weak_notify_cb (gpointer  user_data,
+                       GObject  *where_the_object_was)
+{
+  g_error ("Window was destroyed when waiting to be shown");
+}
+
 static void
 on_window_shown (MetaWindow       *window,
                  WaitForShownData *data)
 {
+  g_object_weak_unref (G_OBJECT (window),
+                       window_weak_notify_cb,
+                       NULL);
   g_main_loop_quit (data->loop);
 }
 
@@ -489,10 +510,13 @@ wait_for_showing_before_redraw (gpointer user_data)
     }
   else
     {
+      g_object_weak_unref (G_OBJECT (data->window),
+                           window_weak_notify_cb,
+                           NULL);
       g_main_loop_quit (data->loop);
     }
 
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 void
@@ -502,6 +526,10 @@ meta_wait_for_window_shown (MetaWindow *window)
   MetaCompositor *compositor = meta_display_get_compositor (display);
   MetaLaters *laters = meta_compositor_get_laters (compositor);
 
+  g_object_weak_ref (G_OBJECT (window),
+                     window_weak_notify_cb,
+                     NULL);
+
   WaitForShownData data = {
     .loop = g_main_loop_new (NULL, FALSE),
     .window = window,
