@@ -7,6 +7,7 @@
 #include <wayland-client.h>
 
 #include "color-management-v1-client-protocol.h"
+#include "color-representation-v1-client-protocol.h"
 #include "cursor-shape-v1-client-protocol.h"
 #include "fractional-scale-v1-client-protocol.h"
 #include "linux-dmabuf-v1-client-protocol.h"
@@ -14,6 +15,8 @@
 #include "test-driver-client-protocol.h"
 #include "viewporter-client-protocol.h"
 #include "xdg-shell-client-protocol.h"
+#include "xdg-toplevel-tag-v1-client-protocol.h"
+#include "xdg-activation-v1-client-protocol.h"
 
 typedef enum _WaylandDisplayCapabilities
 {
@@ -37,20 +40,26 @@ typedef struct _WaylandDisplay
 
   WaylandDisplayCapabilities capabilities;
 
+  GSource *source;
+
   struct wl_display *display;
   struct wl_registry *registry;
   struct wl_compositor *compositor;
   struct wl_subcompositor *subcompositor;
   struct wl_shm *shm;
   struct zwp_linux_dmabuf_v1 *linux_dmabuf;
+  struct wp_color_representation_manager_v1 *color_representation;
   struct wp_fractional_scale_manager_v1 *fractional_scale_mgr;
   struct wp_single_pixel_buffer_manager_v1 *single_pixel_mgr;
   struct wp_color_manager_v1 *color_management_mgr;
   struct wp_cursor_shape_manager_v1 *cursor_shape_mgr;
   struct wp_viewporter *viewporter;
   struct xdg_wm_base *xdg_wm_base;
+  struct xdg_toplevel_tag_manager_v1 *toplevel_tag_manager;
+  struct xdg_activation_v1 *xdg_activation;
   struct wl_seat *wl_seat;
   struct wl_pointer *wl_pointer;
+  struct wl_keyboard *wl_keyboard;
   struct test_driver *test_driver;
 
   gboolean needs_roundtrip;
@@ -86,6 +95,8 @@ typedef struct _WaylandSurface
   GHashTable *pending_state;
   GHashTable *current_state;
 
+  uint32_t last_serial;
+
   int default_width;
   int default_height;
   int width;
@@ -93,9 +104,11 @@ typedef struct _WaylandSurface
   gboolean fixed_size;
 
   uint32_t color;
-  gboolean is_opaque;
 
   int32_t preferred_buffer_scale;
+
+  gboolean manual_paint;
+  gboolean has_alpha;
 } WaylandSurface;
 
 #define WAYLAND_TYPE_SURFACE (wayland_surface_get_type ())
@@ -128,10 +141,10 @@ WaylandSurface * wayland_surface_new_unassigned (WaylandDisplay *display);
 gboolean wayland_surface_has_state (WaylandSurface          *surface,
                                     enum xdg_toplevel_state  state);
 
-void wayland_surface_set_opaque (WaylandSurface *surface);
-
 void wayland_surface_fixate_size (WaylandSurface *surface);
 
+void wayland_surface_commit (WaylandSurface *surface);
+
 void draw_surface (WaylandDisplay    *display,
                    struct wl_surface *surface,
                    int                width,
@@ -150,6 +163,9 @@ void wait_for_effects_completed (WaylandDisplay    *display,
 void wait_for_window_shown (WaylandDisplay    *display,
                             struct wl_surface *surface);
 
+void wait_for_window_configured (WaylandDisplay *display,
+                                 WaylandSurface *surface);
+
 void wait_for_view_verified (WaylandDisplay *display,
                              int             sequence);
 
