@@ -14,21 +14,16 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Written by:
  *     Jonas Ådahl <jadahl@gmail.com>
  */
 
-#ifndef META_WAYLAND_LEGACY_FOREIGN_H
-#define META_WAYLAND_LEGACY_FOREIGN_H
+#pragma once
 
 #include <glib.h>
 
 #include "wayland/meta-wayland-types.h"
 
 gboolean meta_wayland_legacy_xdg_foreign_init (MetaWaylandCompositor *compositor);
-
-#endif /* META_WAYLAND_LEGACY_FOREIGN_H */
