@@ -297,9 +297,6 @@ meta_wayland_pointer_constraint_new (MetaWaylandSurface                       *s
   MetaWaylandPointerConstraint *constraint;
 
   constraint = g_object_new (META_TYPE_WAYLAND_POINTER_CONSTRAINT, NULL);
-  if (!constraint)
-    return NULL;
-
   constraint->surface = surface;
   constraint->seat = seat;
   constraint->lifetime = lifetime;
@@ -917,11 +914,6 @@ init_pointer_constraint (struct wl_resource                       *resource,
                                                     region,
                                                     lifetime,
                                                     cr);
-  if (constraint == NULL)
-    {
-      wl_client_post_no_memory (client);
-      return;
-    }
 
   surface_add_pointer_constraint (surface, constraint);
 
@@ -1008,31 +1000,40 @@ static const struct zwp_locked_pointer_v1_interface locked_pointer_interface = {
 
 static MetaWaylandSurface *
 pointer_constraints_get_focus_surface (MetaWaylandEventHandler *handler,
-                                       ClutterInputDevice      *device,
-                                       ClutterEventSequence    *sequence,
+                                       ClutterFocus            *focus,
                                        gpointer                 user_data)
 {
   return meta_wayland_event_handler_chain_up_get_focus_surface (handler,
-                                                                device,
-                                                                sequence);
+                                                                focus);
 }
 
 static void
 pointer_constraints_focus (MetaWaylandEventHandler *handler,
-                           ClutterInputDevice      *device,
-                           ClutterEventSequence    *sequence,
+                           ClutterFocus            *focus,
                            MetaWaylandSurface      *surface,
                            gpointer                 user_data)
 {
   MetaWaylandPointerConstraint *constraint = user_data;
 
-  if (!sequence &&
-      (clutter_input_device_get_capabilities (device) &
-       CLUTTER_INPUT_CAPABILITY_POINTER) &&
-      surface != constraint->surface)
-    meta_wayland_pointer_constraint_deactivate (constraint);
-  else
-    meta_wayland_event_handler_chain_up_focus (handler, device, sequence, surface);
+  if (CLUTTER_IS_SPRITE (focus))
+    {
+      ClutterInputDevice *device;
+      ClutterEventSequence *sequence;
+
+      device = clutter_sprite_get_device (CLUTTER_SPRITE (focus));
+      sequence = clutter_sprite_get_sequence (CLUTTER_SPRITE (focus));
+
+      if (!sequence &&
+          (clutter_input_device_get_capabilities (device) &
+           CLUTTER_INPUT_CAPABILITY_POINTER) &&
+          surface != constraint->surface)
+        {
+          meta_wayland_pointer_constraint_deactivate (constraint);
+          return;
+        }
+    }
+
+  meta_wayland_event_handler_chain_up_focus (handler, focus, surface);
 }
 
 static const MetaWaylandEventInterface pointer_constraints_event_interface = {
